@@ -4,17 +4,17 @@
 #' @param rho_mat Matrix of true-score correlations.
 #' @param mu_vec Vector of means.
 #' @param sigma_vec Vector of observed-score standard deviations.
-#' @param rel_vec Vector of reliabilities corresponding to the variables in \code{rho_mat.}
-#' @param sr_vec Vector of selection ratios corresponding to the variables in \code{rho_mat}
+#' @param rel_vec Vector of reliabilities corresponding to the variables in \code{rho_mat}.
+#' @param sr_vec Vector of selection ratios corresponding to the variables in \code{rho_mat}.
 #' (set selection ratios to 1 for variables that should not be used in selection).
 #' @param k_items_vec Number of test items comprising each of the variables to be simulated (all are single-item variables by default).
-#' @param wt_mat Optional matrix of weights to use in forming a composite of the variables in \code{rho_mat.} Matrix should have as many rows (or vector elements) as there are variables in \code{rho_mat}.
+#' @param wt_mat Optional matrix of weights to use in forming a composite of the variables in \code{rho_mat}. Matrix should have as many rows (or vector elements) as there are variables in \code{rho_mat}.
 #' @param sr_composites Optional vector selection ratios for composite variables. If not \code{NULL}, \code{sr_composites} must have as many elements as there are columns in \code{wt_mat}.
 #' @param var_names Vector of variable names corresponding to the variables in \code{rho_mat}.
 #' @param composite_names Optional vector of names for composite variables.
 #' @param ... Further arguments.
 #'
-#' @return A list of observed-score, true-score, and error-score data frames. If selection is requested, the data frames will include logical variables indicating whether each case would be selected on the basis of observed scores, true scores, and error scores.
+#' @return A list of observed-score, true-score, and error-score data frames. If selection is requested, the data frames will include logical variables indicating whether each case would be selected on the basis of observed scores, true scores, or error scores.
 #' @export
 #'
 #' @importFrom MASS mvrnorm
@@ -125,8 +125,10 @@ simulate_psych <- function(n, rho_mat,
           error <- error[,keep_id]
      }
 
-     out <- list(observed = obs, true = true, error = error)
-     class(out) <- c("psychmeta", "simulate_psych")
+     out <- list(observed = as_tibble(obs), 
+                 true = as_tibble(true),
+                 error = as_tibble(error))
+     class(out) <- "simdat_psych"
      out
 }
 
